package me.km.blockprotections;

import me.km.KajetansMod;
import me.km.module.Module;
import me.km.utils.Location;
import me.km.utils.Utils;
import net.minecraft.block.Block;
import net.minecraft.block.ContainerBlock;
import net.minecraft.block.DoorBlock;
import net.minecraft.block.BlockState;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.block.Blocks;
import net.minecraft.state.properties.DoubleBlockHalf;
import net.minecraft.util.Hand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.world.IWorld;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.eventbus.api.EventPriority;
import net.minecraftforge.eventbus.api.SubscribeEvent;

public class BlockProtection extends Module
{
    private final static String BLOCK_BYPASS = "block.bypass";
    
    private final IBlockProtection bank;
    
    public BlockProtection() 
    {
        super("Blocks", TextFormatting.BLUE);
        if(KajetansMod.databank.isDummyDatabank())
        {
            bank = new DummyBlockProtection();
        }
        else
        {
            bank = new BlockProtectionBank();
        }
        registerEvent(this);
    }
    
    public IBlockProtection getDatabank()
    {
        return bank;
    }
    
    private BlockPos getSameNearbyBlock(IWorld w, BlockPos pos, Block b)
    {           
        Location l = new Location(w, pos);
        if(l.getRelativeBlockState(1, 0, 0).getBlock() == b)
        {
            return pos.add(1, 0, 0);
        }
        else if(l.getRelativeBlockState(-1, 0, 0).getBlock() == b)
        {
            return pos.add(-1, 0, 0);
        }
        else if(l.getRelativeBlockState(0, 0, 1).getBlock() == b)
        {
            return pos.add(0, 0, 1);
        }
        else if(l.getRelativeBlockState(0, 0, -1).getBlock() == b)
        {
            return pos.add(0, 0, -1);
        }
        return null;
    }
    
    @SubscribeEvent(priority = EventPriority.HIGHEST)
    public void onBlockPlace(BlockEvent.EntityPlaceEvent e)
    {
        Block b = e.getPlacedBlock().getBlock();
        if(!(e.getEntity() instanceof PlayerEntity))
        {
            return;
        }
        PlayerEntity p = (PlayerEntity) e.getEntity();
        IWorld w = e.getWorld();
        
        if(b == Blocks.CHEST || b == Blocks.TRAPPED_CHEST) // Deny placing chests near other protected chests
        {
            BlockPos otherChest = getSameNearbyBlock(w, e.getPos(), b);
            if(otherChest != null && !bank.hasAccess(otherChest, w, p) && !KajetansMod.perms.hasPermission(p, BLOCK_BYPASS))
            {
                e.setCanceled(true);
            }        
        } 
        else if(b == Blocks.HOPPER) // Deny placing of hoppers under blocks
        {
            BlockPos pos = e.getPos().add(0, 1, 0);
            if(w.getBlockState(pos).getBlock() instanceof ContainerBlock)
            {
                if(!bank.hasAccess(pos, w, p) && !KajetansMod.perms.hasPermission(p, BLOCK_BYPASS))
                {
                    e.setCanceled(true);
                }               
            }            
        }
    }
    
    @SubscribeEvent(priority = EventPriority.HIGHEST)
    public void onBlockBreak(BlockEvent.BreakEvent e)
    {
        PlayerEntity p = e.getPlayer();
        BlockState state = e.getState();

        BlockPos pos = e.getPos();
        if(Utils.getStateValue(state, DoorBlock.HALF) == DoubleBlockHalf.UPPER)
        {
            pos = pos.add(0, -1, 0);
        } 
        IWorld w = e.getWorld();
        if(bank.hasAccess(pos, w, p) || KajetansMod.perms.hasPermission(p, BLOCK_BYPASS))
        {
            // TODO remove protection
            return;
        }   
        e.setCanceled(true);          
    }
    
    @SubscribeEvent(priority = EventPriority.HIGHEST)
    public void onContainerOpen(PlayerInteractEvent.RightClickBlock e)
    {
        if(e.getHand() != Hand.MAIN_HAND)
        {
            return;
        }
        BlockState state = e.getWorld().getBlockState(e.getPos());
        PlayerEntity p = e.getEntityPlayer();
        BlockPos pos = e.getPos();
        if(Utils.getStateValue(state, DoorBlock.HALF) == DoubleBlockHalf.UPPER)
        {
            pos = pos.add(0, -1, 0);
        }
        if(!bank.hasAccess(pos, e.getWorld(), p) && !KajetansMod.perms.hasPermission(p, BLOCK_BYPASS))
        {
            e.setCanceled(true);   
        }  
    }
}
