package me.km.effects;

import me.km.KajetansMod;
import me.km.utils.Utils;
import me.km.entities.EntityItemProjectile;
import net.minecraft.entity.Entity;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.item.EnderPearlEntity;
import net.minecraft.entity.item.ExperienceBottleEntity;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.entity.projectile.*;
import net.minecraft.inventory.EquipmentSlotType;
import net.minecraft.item.Items;
import net.minecraft.item.ItemStack;
import net.minecraft.particles.IParticleData;
import net.minecraft.potion.Effect;
import net.minecraft.potion.EffectInstance;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionUtils;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.datafix.fixes.TippedArrow;
import net.minecraft.util.math.Vec3d;
import net.minecraft.world.World;
import net.minecraft.world.ServerWorld;

public class EffectUtils
{
    // -----------------------------------------------------------------------------------
    // Particles
    // -----------------------------------------------------------------------------------

    public static void spawnParticle(ServerWorld w, IParticleData particle, double x, double y, double z, double offX, double offY, double offZ, int count, int... data)
    {
        //w.spawnParticle(particle, false, x, y, z, count, offX, offY, offZ, 1, data);
    }

    public static void spawnParticle(ServerWorld w, IParticleData particle, double x, double y, double z, double offX, double offY, double offZ, int count)
    {
        spawnParticle(w, particle, x, y, z, offX, offY, offZ, count, new int[0]);
    }

    public static void spawnParticle(ServerWorld w, IParticleData particle, double x, double y, double z, int count)
    {
        spawnParticle(w, particle, x, y, z, 0, 0, 0, count);
    }

    public static void spawnParticle(ServerWorld w, IParticleData particle, Entity ent, int count)
    {
        spawnParticle(w, particle, ent.posX, ent.posY, ent.posZ, count);
    }

    public static void spawnParticleCircle(ServerWorld w, double x, double y, double z, IParticleData particle, double radius, int counter, int... data)
    {
        double angle = 2 * Math.PI / counter;
        for(int i = 0; i < counter; i++)
        {
            spawnParticle(w, particle, x + Math.cos(i * angle) * radius, y, z + Math.sin(i * angle) * radius, 0, 0, 0, 1, data);
        }
    }

    public static void spawnParticleCircle(ServerWorld w, Entity ent, IParticleData particle, double radius, int counter, int... data)
    {
        spawnParticleCircle(w, ent.posX, ent.posY, ent.posZ, particle, radius, counter, data);
    }

    public static void spawnParticleCircle(ServerWorld w, double x, double y, double z, IParticleData particle, double radius, int counter)
    {
        spawnParticleCircle(w, x, y, z, particle, radius, counter, new int[0]);
    }

    public static void spawnParticleCircle(ServerWorld w, Entity ent, IParticleData particle, double radius, int counter)
    {
        spawnParticleCircle(w, ent.posX, ent.posY, ent.posZ, particle, radius, counter, new int[0]);
    }

    // -----------------------------------------------------------------------------------
    // Sounds
    // -----------------------------------------------------------------------------------
    public static void playSound(World w, SoundEvent se, SoundCategory sc, double x, double y, double z)
    {
        w.playSound(null, x, y, z, se, sc, 1, w.rand.nextFloat() * 0.1f + 0.9f);
    }

    public static void playSound(PlayerEntity p, SoundEvent se, SoundCategory sc)
    {
        playSound(p.world, se, sc, p.posX, p.posY, p.posZ);
    }

    public static void playSound(PlayerEntity p, SoundEvent se)
    {
        playSound(p.world, se, SoundCategory.PLAYERS, p.posX, p.posY, p.posZ);
    }

    // -----------------------------------------------------------------------------------
    // Potions
    // -----------------------------------------------------------------------------------
    public static void addPotionTo(LivingEntity ent, Effect potion, int duration, int amplifier)
    {
        if(ent.isPotionActive(potion))
        {
            ent.removePotionEffect(potion);
        }
        ent.addPotionEffect(new EffectInstance(potion, duration, amplifier));
    }

    // -----------------------------------------------------------------------------------
    // Projectiles
    // -----------------------------------------------------------------------------------
    public static <T> T launchProjectile(PlayerEntity p, Class<? extends T> projectile, double scale, Object data)
    {
        World w = p.world;
        Entity launch = null;

        if(EntityItemProjectile.class == projectile)
        {
            if(data == null)
            {
                throw new NullPointerException("Data musn't be null for EntityItemProjectile");
            }
            ItemStack stack = (ItemStack) data;
            if(stack.isEmpty())
            {
                throw new IllegalArgumentException("Empty ItemStack not allowed here");
            }
            launch = new EntityItemProjectile(p, stack.copy());
            ((EntityItemProjectile) launch).setHeadingFromThrower(p, p.rotationPitch, p.rotationYaw, 0.0f, 1.5f, 1.0f);
        }
        else if(SnowballEntity.class == projectile)
        {
            launch = new SnowballEntity(w, p);
            ((SnowballEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, 0.0f, 1.5f, 1.0f);
        }
        else if(EggEntity.class == projectile)
        {
            launch = new EggEntity(w, p);
            ((EggEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, 0.0f, 1.5f, 1.0f);
        }
        else if(EnderPearlEntity.class == projectile)
        {
            launch = new EnderPearlEntity(w, p);
            ((EnderPearlEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, 0.0f, 1.5f, 1.0f);
        }
        else if(PotionEntity.class == projectile)
        {
            launch = new PotionEntity(w, p);
            ((PotionEntity) launch).setItem((ItemStack) data);
            ((PotionEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, -20.0f, 0.5f, 1.0f);
        }
        else if(ExperienceBottleEntity.class == projectile)
        {
            launch = new ExperienceBottleEntity(w, p);
            ((ExperienceBottleEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, -20.0f, 0.7f, 1.0f);
        }
        else if(AbstractArrowEntity.class.isAssignableFrom(projectile))
        {
            if(SpectralArrowEntity.class == projectile)
            {
                launch = new SpectralArrowEntity(w, p);
            }
            else
            {
                launch = new ArrowEntity(w, p);
                if(data != null)
                {
                    ((ArrowEntity) launch).setPotionEffect((ItemStack) data);
                }
            }
            ((AbstractArrowEntity) launch).shoot(p, p.rotationPitch, p.rotationYaw, 0.0F, 3.0F, 1.0F);
        }
        else if(DamagingProjectileEntity.class.isAssignableFrom(projectile))
        {
            Vec3d v = p.getLookVec().scale(10);
            if(SmallFireballEntity.class == projectile)
            {
                launch = new SmallFireballEntity(w, p, v.x, v.y, v.z);
            }
            else if(WitherSkullEntity.class == projectile)
            {
                launch = new WitherSkullEntity(w, p, v.x, v.y, v.z);
            }
            else if(DragonFireballEntity.class == projectile)
            {
                launch = new DragonFireballEntity(w, p, v.x, v.y, v.z);
            }
            else
            {
                launch = new FireballEntity(w, p, v.x, v.y, v.z);
            }
        }

        Utils.scaleVelocity(launch, scale);
        w.addEntity(launch);
        return (T) launch;
    }

    public static ArrowEntity launchTippedArrow(PlayerEntity p, double scale, Effect potion, int duration, int amplifier)
    {
        ItemStack stack = new ItemStack(Items.TIPPED_ARROW);
        PotionUtils.addPotionToItemStack(stack, new Potion(new EffectInstance(potion, duration, amplifier)));
        ArrowEntity arrow = launchProjectile(p, ArrowEntity.class, scale, stack);
        return arrow;
    }

    public static void jumpTo(LivingEntity j, LivingEntity g)
    {
        Utils.setVelocity(j, (g.posX - g.posX) * 0.2, (g.posY - g.posY) * 0.2 + 0.9, (g.posZ - g.posZ) * 0.2);
        KajetansMod.scheduler.scheduleTask(() -> 
        {
            Utils.setVelocity(j, (g.posX - j.posX) * 0.2, (g.posY - j.posY) * 0.2, (g.posZ - j.posZ) * 0.2);
        }, 12);
    }
}
