package me.km.entities;

import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.function.Consumer;
import javax.imageio.ImageIO;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.DownloadImageBuffer;
import net.minecraft.client.renderer.texture.DynamicTexture;
import net.minecraft.client.renderer.texture.NativeImage;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.api.distmarker.Dist;
import net.minecraftforge.api.distmarker.OnlyIn;

@OnlyIn(Dist.CLIENT)
public class HumanSkinLoader
{
    public final static HumanSkinLoader INSTANCE = new HumanSkinLoader();
    
    private static final ResourceLocation TEXTURE_STEVE = new ResourceLocation("textures/entity/steve.png");
    
    private final HashMap<String, ResourceLocation> skins;
    private final HashSet<String> loading;
    
    private final HashMap<String, ArrayList<Consumer<ResourceLocation>>> queue;
    
    private final TextureManager manager;
    private final DownloadImageBuffer converter;
    
    public HumanSkinLoader()
    {
        this.skins = new HashMap<>();
        this.loading = new HashSet<>();
        this.queue = new HashMap<>();
        this.manager = Minecraft.getInstance().getRenderManager().textureManager;
        this.converter = new DownloadImageBuffer();
    }
    
    public ResourceLocation getTexture(String name, Consumer<ResourceLocation> delayed)
    {
        ResourceLocation loc = skins.get(name);
        if(loc == null)
        {
            if(loading.add(name))
            {
                downloadSkin(name, image -> 
                {
                    ResourceLocation rloc = manager.getDynamicTextureLocation("skin_" + name, new DynamicTexture(image));
                    skins.put(name, rloc);
                    loading.remove(name);
                    delayed.accept(rloc);
                    
                    ArrayList<Consumer<ResourceLocation>> list = queue.get(name);
                    if(list != null)
                    {
                        list.forEach(c -> c.accept(rloc));
                        list.clear();
                        queue.remove(name);
                    }
                });
            }
            else
            {
                ArrayList<Consumer<ResourceLocation>> list = queue.get(name);
                if(list == null)
                {
                    list = new ArrayList<>();
                    queue.put(name, list);
                }
                list.add(delayed);
            }
            return TEXTURE_STEVE;
        }
        return loc;
    }
    
    private void downloadSkin(String name, Consumer<NativeImage> delayed)
    {
        new Thread(() -> 
        {
            HttpURLConnection httpurlconnection = null;
            try
            {
                //httpurlconnection = (HttpURLConnection)(new URL("http://skins.minecraft.net/MinecraftSkins/" + name + ".png"))
                httpurlconnection = (HttpURLConnection) (new URL("http://skins.hammerle.me/" + name + ".png"))
                        .openConnection(Minecraft.getInstance().getProxy());
                httpurlconnection.setDoInput(true);
                httpurlconnection.setDoOutput(false);
                httpurlconnection.connect();

                int code = httpurlconnection.getResponseCode();
                if(code != 200)
                {
                    // Failed own server, trying mojangs
                    //System.out.println("https://crafatar.com/skins/" + name + ".png");
                    httpurlconnection.disconnect();
                    httpurlconnection = (HttpURLConnection) (new URL("https://crafatar.com/skins/" + name + ".png"))
                            .openConnection(Minecraft.getInstance().getProxy());
                    httpurlconnection.setDoInput(true);
                    httpurlconnection.setDoOutput(false);
                    httpurlconnection.connect();
                    code = httpurlconnection.getResponseCode();
                    if(code != 200)
                    {
                        System.out.println("Server response code did return " + code + ", skin servers might be down.");
                        return;
                    }
                }
                
                NativeImage image = converter.parseUserSkin(NativeImage.read(httpurlconnection.getInputStream()));
                Minecraft.getInstance().enqueue(() -> delayed.accept(image));
            }
            catch (Exception ex)
            {
                System.out.println("Error occurred when downloading skin, however, skin servers seem to be up.");
            }
            finally
            {
                if (httpurlconnection != null)
                {
                    httpurlconnection.disconnect();
                }
            }
        }).start();
    }
}