package me.km.module;

import me.km.utils.Utils;
import java.lang.reflect.InvocationTargetException;
import me.km.KajetansMod;
import me.km.PackageHelper;
import net.minecraft.command.ICommandSource;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.util.text.ITextComponent;
import net.minecraft.util.text.StringTextComponent;
import net.minecraft.util.text.TextFormatting;
import net.minecraft.world.World;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.event.server.FMLServerStartingEvent;

public class Module
{
    private final String prefix;
    private final String color;
    
    public Module(String prefix, TextFormatting color)
    {
        this.color = color.toString();
        this.prefix = String.format("[%s%s§r] ", color, prefix);
        sendToConsole(String.format("Module %swas loaded.", this.prefix));
    }
    
    //--------------------------------------------------------------------------
    // messages
    //--------------------------------------------------------------------------

    public final void send(ICommandSource cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        cs.sendMessage(new StringTextComponent(prefix + msg));
    }

    public final void sendToPlayers(World w, double x, double y, double z, double radius, String msg)
    {
        ITextComponent s = new StringTextComponent(prefix + msg);
        Utils.getEntities(w, x, y, z, radius, PlayerEntity.class).forEach(p -> ((PlayerEntity) p).sendMessage(s));
    }
    
    public final void sendBroadcast(String msg)
    {
        KajetansMod.server.getPlayerList().sendMessage(new StringTextComponent(prefix + msg));
    }
    
    public final void sendListElement(ICommandSource cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        cs.sendMessage(new StringTextComponent(String.format(" %s-§r %s", color, msg)));
    } 
    
    public final void sendHelpListElement(ICommandSource cs, String msg, String msg2)
    {
        if(cs == null)
        {
            return;
        }
        cs.sendMessage(new StringTextComponent(String.format(" %s- %s§r %s", color, msg, msg2)));
    }
    
    public final void sendWarning(ICommandSource cs, String msg)
    {
        if(cs == null)
        {
            return;
        }
        cs.sendMessage(new StringTextComponent(String.format("%s%s%s", prefix, TextFormatting.RED, msg)));
    }
   
    public final void sendToConsole(String msg)
    {
        send(KajetansMod.server, msg);
    }
    
    public final void sendListElementToConsole(String msg)
    {
        sendListElement(KajetansMod.server, msg);
    }
    
    public final void sendHelpListElementToConsole(String msg, String msg2)
    {
        sendHelpListElement(KajetansMod.server, msg, msg2);
    }
    
    public final void sendWarningToConsole(String msg)
    {
        sendWarning(KajetansMod.server, msg);
    }
    
    @SuppressWarnings("unchecked")
    public void registerCommands(FMLServerStartingEvent e, String packageName)
    {
        PackageHelper.getClasses(packageName).stream().filter(c -> ModuleCommand.class.isAssignableFrom(c)).forEach((Class c) -> 
        {
            try 
            {
                ModuleCommand command = (ModuleCommand) c.getDeclaredConstructor(Module.class).newInstance(this);
                KajetansMod.commands.registerCommand(command);
            }
            catch(ClassCastException | NoSuchMethodException | SecurityException | InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex)
            { 
                this.sendToConsole("Command '" + c + "' has thrown an error.");
            }
        });
        this.sendToConsole("All commands were loaded.");
    }
 
    @SuppressWarnings("unchecked")
    public void registerEvents(String packageName)
    {
        PackageHelper.getClasses(packageName).stream().filter(c -> ModuleListener.class.isAssignableFrom(c)).forEach((Class c) -> 
        {
            try 
            {   
                MinecraftForge.EVENT_BUS.register((ModuleListener) c.getDeclaredConstructor(Module.class).newInstance(this));
            }
            catch(NoSuchMethodException | ClassCastException | SecurityException | InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex)
            {
                this.sendToConsole("Error in event class " + c + " " + ex.toString());
            }
        });
        this.sendToConsole("All events were loaded.");
    }
    
    public static void registerEvent(Object o)
    {
        MinecraftForge.EVENT_BUS.register(o);
    }
}
