package me.km.networking;

import java.nio.charset.StandardCharsets;
import java.util.function.Supplier;
import me.km.entities.EntityHuman;
import me.km.entities.HumanSkinLoader;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.Entity;
import net.minecraft.network.PacketBuffer;
import net.minecraftforge.fml.network.NetworkEvent;

public class HumanUpdate
{
    private int humanId;
    private String name;
    
    public HumanUpdate() 
    {
        humanId = -1;
        name = null;
    }
    
    public HumanUpdate(EntityHuman h) 
    {
        humanId = h.getEntityId();
        name = h.getSkinName();
    }
    
    public static void writeBytes(HumanUpdate hu, PacketBuffer buf)
    {
        buf.writeInt(hu.humanId);
        byte[] b = hu.name.getBytes(StandardCharsets.UTF_8);
        buf.writeInt(b.length);
        buf.writeBytes(b);
    }
    
    public static HumanUpdate fromBytes(PacketBuffer buf)
    {
        HumanUpdate hu = new HumanUpdate();
        hu.humanId = buf.readInt();
        int length = buf.readInt();
        hu.name = buf.readBytes(length).toString(StandardCharsets.UTF_8);
        return hu;
    }
    
    public static void handle(HumanUpdate hu, Supplier<NetworkEvent.Context> context)
    {
        context.get().enqueueWork(() -> 
        {
            Entity ent = Minecraft.getInstance().world.getEntityByID(hu.humanId);
            if(ent == null || !(ent instanceof EntityHuman))
            {
                return;
            }
            EntityHuman h = (EntityHuman) ent;
            h.setSkinName(hu.name);
            h.setTexture(HumanSkinLoader.INSTANCE.getTexture(hu.name, loc -> h.setTexture(loc)));
        });
        context.get().setPacketHandled(true);
    }
}
