package me.km.scoreboard;

import java.util.HashMap;
import java.util.HashSet;
import java.util.UUID;
import me.km.KajetansMod;
import net.minecraft.entity.player.ServerPlayerEntity;
import net.minecraft.network.play.server.SDisplayObjectivePacket;
import net.minecraft.network.play.server.SScoreboardObjectivePacket;
import net.minecraft.network.play.server.SUpdateScorePacket;
import net.minecraft.scoreboard.ScoreCriteria;
import net.minecraft.scoreboard.ScoreCriteria.RenderType;
import net.minecraft.scoreboard.ScoreObjective;
import net.minecraft.scoreboard.Scoreboard;
import net.minecraft.scoreboard.ServerScoreboard.Action;
import net.minecraft.util.text.StringTextComponent;

public class PlayerScoreboard 
{
    private final HashMap<Integer, String> elements = new HashMap<>();
    private final HashSet<Integer> toRemove = new HashSet<>();
    private final HashMap<Integer, String> toAdd = new HashMap<>();

    private final Scoreboard board = new Scoreboard();
    private final ScoreObjective o;
    private boolean changeNeeded = true;

    private final UUID uuid;

    public PlayerScoreboard(ServerPlayerEntity p)
    {
        o = new ScoreObjective(board, "ScoreBoardAPI", ScoreCriteria.DUMMY, new StringTextComponent("§6---------------"), RenderType.INTEGER);
        this.uuid = p.getUniqueID();
        p.connection.sendPacket(new SScoreboardObjectivePacket(o, 0));
    }

    public void addText(int id, String text)
    { 
        String s = elements.get(id);
        if(s != null && !s.equals(text))
        {
            toRemove.add(id);
            toAdd.put(id, text);
            changeNeeded = true;
        }
        else if(s == null)
        {
            toAdd.put(id, text);
            changeNeeded = true;
        }
    }

    public void removeText(int id)
    { 
        if(elements.containsKey(id))
        {
            toRemove.add(id);
            changeNeeded = true;
        }
    }

    public void clear()
    {
        elements.keySet().forEach(i -> toRemove.add(i));
        update();
    }

    public boolean update()
    {         
        if(!changeNeeded)
        {
            return false;
        }

        ServerPlayerEntity p = KajetansMod.server.getPlayerList().getPlayerByUUID(uuid);

        if(p == null)
        {
            return true;
        }

        if(!toRemove.isEmpty())
        {
            toRemove.forEach(i -> 
            {
                String s = elements.remove(i);
                if(s != null)
                {
                    p.connection.sendPacket(new SUpdateScorePacket(Action.REMOVE, null, null, -1));
                }
            });
            toRemove.clear();
        }

        if(!toAdd.isEmpty())
        {
            toAdd.entrySet().stream().forEach((e) -> 
            {
                elements.put(e.getKey(), e.getValue());
                p.connection.sendPacket(new SUpdateScorePacket(Action.CHANGE, o.getName(), e.getValue(), e.getKey()));
            });      
            toAdd.clear();
        }

        changeNeeded = false;
        // displaying objective in sidebar
        p.connection.sendPacket(new SDisplayObjectivePacket(1, o));
        return false;
    }
}
