package me.km.snuviscript;

import java.util.ArrayList;
import java.util.Arrays;
import me.km.KajetansMod;
import me.km.module.Module;
import java.util.Collection;
import me.hammerle.snuviscript.code.Script;
import me.hammerle.snuviscript.inputprovider.Variable;
import me.km.module.ModuleCommand;
import me.km.utils.Utils;
import me.km.exception.PlayerNotFoundException;
import net.minecraft.command.ICommandSource;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.ServerPlayerEntity;

public class CommandScript extends ModuleCommand
{
    public CommandScript(Module m) 
    {
        super(m);
    }

    @Override
    public String getName()
    {
        return "script";
    }
    
    @Override
    public Iterable<String> getAliases()
    {
        ArrayList<String> list = new ArrayList<>();
        list.add("s");
        return list;
    }
    

    @SuppressWarnings("unchecked")
    @Override
    public void execute(ICommandSource cs, String[] arg) 
    {
        Module m = this.getModule();
        if(arg.length >= 1)
        {
            switch(arg[0].toLowerCase())
            {
                case "s":
                case "start":
                {
                    if(arg.length >= 2)
                    {
                        String[] pars = Arrays.copyOfRange(arg, 1, arg.length);
                        KajetansMod.scripts.startScript(pars);
                        return;
                    }
                    break;
                }
                case "startp":
                {
                    if(arg.length >= 3)
                    {
                        try
                        {
                            PlayerEntity p = Utils.getPlayerByName(arg[1]);
                            String[] pars = Arrays.copyOfRange(arg, 2, arg.length);
                            KajetansMod.scripts.startPlayerScript(p, pars);
                        }
                        catch(PlayerNotFoundException ex)
                        {
                            m.send(cs, "Cannot find player '" + arg[1] + "'.");
                        }
                        return;
                    }
                    break;
                }
                case "v":
                case "variable":    
                {
                    if(arg.length >= 3)
                    {
                        try
                        {
                            Script sc = KajetansMod.scripts.getScriptManager().getScript(Integer.parseInt(arg[1]));
                            if(sc == null)
                            {
                                throw new NumberFormatException();
                            }
                            Variable var = sc.getVar(arg[2]);
                            if(var != null)
                            {
                                m.send(cs, arg[2] + " = " + var.get(sc));
                            }
                            else
                            {
                                m.send(cs, arg[2] + " = null");
                            }
                        }
                        catch(NumberFormatException ex)
                        {
                            m.send(cs, "'" + arg[1] + "' ist keine gültige ID.");
                        }
                        return;
                    }
                    break;
                }
                case "see":
                {
                    Collection<Script> scripts = KajetansMod.scripts.getScriptManager().getScripts();
                    if(scripts.isEmpty())
                    {
                        m.send(cs, "Momentan sind keine Scripts aktiv.");
                        return;
                    }
                    m.send(cs, "Folgende Scripts sind aktiv:");
                    scripts.forEach(sc ->
                    {
                        m.sendHelpListElement(cs, sc.getName(), "(" + sc.getId() + ")");
                        KajetansMod.scripts.getPlayerList(sc.getId()).forEach(uuid -> 
                        {
                            ServerPlayerEntity p = KajetansMod.server.getPlayerList().getPlayerByUUID(uuid);
                            m.sendListElement(cs, p == null ? "null" : p.getName().getFormattedText());
                        });
                    });
                    return;
                }
                case "term":
                {
                    if(arg.length >= 2)
                    {
                        try
                        {
                            if(arg[1].equals("all"))
                            {
                                KajetansMod.scripts.clearPlayerRegistry();
                                if(KajetansMod.scripts.getScriptManager().removeScriptsSafe())
                                {
                                    m.send(cs, "Alle aktiven Scripts wurden beendet.");
                                }
                                else
                                {
                                    m.send(cs, "Iterating is not allowed currently");
                                }
                                return;
                            }
                            int id = Integer.parseInt(arg[1]);
                            Script qd = KajetansMod.scripts.getScriptManager().getScript(id);
                            if(qd != null)
                            {
                                KajetansMod.scripts.getScriptManager().removeScriptSafe(qd);
                                m.send(cs, "Das Script '" + qd.getName() + "' wurde beendet.");
                            }
                            else
                            {
                                m.send(cs, String.format("Script-ID '%d' is not valid", id));
                            }
                        }
                        catch(NumberFormatException ex)
                        {
                            m.send(cs, "'" + arg[1] + "' ist keine gültige ID.");
                        } 
                        catch(Exception ex)
                        {
                            m.send(cs, "Beim Beenden eines Scripts trat ein Fehler auf.");
                            ex.printStackTrace();
                        } 
                        return;
                    }
                    break;
                }
            }
        }
        
        m.send(cs, "/script ...");
        m.sendHelpListElement(cs, "start <scripts...>", "Startet ein Script");
        m.sendHelpListElement(cs, "startp <player> <scripts...>", "Startet ein Script");
        m.sendHelpListElement(cs, "variable <id> <name>", "Zeigt den Wert einer Variable");       
        m.sendHelpListElement(cs, "see", "Gibt alle aktiven Scripts aus");
        m.sendHelpListElement(cs, "term <id/all>", "Entfernt Scripts aus dem RAM");
    }
}

